local mod = RW:Boss("Omnotron Defense System", "Blackwing Descent")
local L

mod.BossIcon = [[Interface\ICONS\achievement_dungeon_blackwingdescent_darkironcouncil]]
mod.SubCatIcon = [[Interface\ICONS\achievement_boss_nefarion]]

---------------------------
-- Variables

-- Electron	
	-- Conducor
	local iConductor, wConductor, tConductorCD
	-- Unstable Shield
	local tUnstableShieldDur, wUnstableShield
		
-- Arcanotron
	-- Power Conversion
	local tPowerConversionDur, wPowerConversion
	-- Power Generator
	local tPowerGeneratorCD, wPowerGenerator

-- Magmatron
	-- Acquire
	local iAquire, wAcquire, tAcquireCD
	-- Barrier
	local tBarrierDur, wBarrier

-- Toxitron
	-- Fixate
	local wFixate, iFixate, mFixate
	-- Poison Protocol
	local wProtocol, tProtocolCD
	-- Shell
	local wShell, tShellDur
	-- Bomb
	local wBomb, tBombCD
	
-- Active
local tActiveDur, wActiveChanged


-- Electron:	42179
-- Arcanotron:	42166
-- Magmatron:	42178
-- Toxitron:	42180

---------------------------
-- Init

function mod:OnRegister()
	L = mod:RegisterLocale(self:GetName(), "enUS", true)
	if L then
		L["Electron"] = true
		L["Arcanotron"] = true
		L["Magmatron"] = true
		L["Tixitron"] = true
		L["Nefarian"] = true
	
		L["Change Target!"] = true
		L["Warn when to switch target"] = true
		
		L["Stop hitting %s!"] = true
		L["Warn when to stop dps on target"] = true
		
		L["Next Nefairan ability"] = true
		L["Timer for next Nefarian ability"] = true
		L["Nefarian Abilities"] = true
	end
	L = mod:GetLocale()
	self.L = L
	
	self:SetTrigger(42178, 42180, 42179, 42166)
	self:SetWin()
	
	self:AddToCategory(L["Electron"], 79888, 79900)
	self:AddToCategory(L["Arcanotron"], 79729, 79624)
	self:AddToCategory(L["Magmatron"], 79501, 79582)
	self:AddToCategory(L["Tixitron"], 80094, 80053, 79835, 80157)
	self:AddToCategory(L["Nefarian"], 92048, 92053, 92023, 91849)
	
	-- Based on Video/Logs every 51 secs a golem goes from active -> shielded
	-- And appears that only 2 golems are up at any given time
	
	-- Electron	
		-- Conducor
		iConductor		= self:Icon("CROSS", 79888, false, 10)
		wConductor		= self:RunawayWarning(79888)
		tConductorCD	= self:CDTimer(25, 79888)
		-- Unstable Shield
		tUnstableShieldDur	= self:DurTimer(10, 79900)
		wUnstableShield		= self:CustomWarning(L["Stop hitting %s!"], L["Warn when to stop dps on target"], 79900)
	
	-- Arcanotron
		-- Power Conversion
		wPowerConversion	= self:CustomWarning(L["Stop hitting %s!"], L["Warn when to stop dps on target"], 79729)
		tPowerConversionDur	= self:DurTimer(10, 79729)
		-- Power Generator
		wPowerGenerator		= self:CastWarning(79624)
		tPowerGeneratorCD	= self:CDTimer(30, 79624)
	
	-- Magmatron
		-- Acquire (Flame)
		iAquire			= self:Icon("DIAMOND", 79501, false, 10)
		wAcquire		= self:RunawayWarning(79501)
		tAcquireCD		= self:CDTimer(40, 79501)		
		-- Barrier
		wBarrier		= self:CustomWarning(L["Stop hitting %s!"], L["Warn when to stop dps on target"], 79582)
		tBarrierDur		= self:DurTimer(10, 79582)
	
	-- Toxitron
		-- Fixate
		wFixate			= self:RunawayWarning(80094)
		iFixate			= self:Icon("STAR", 80094, false, 10, "CIRCLE", "SQUARE")
		mFixate			= self:TargetMessage(80094)		
		-- Poison Protocol
		wProtocol		= self:CastWarning(80053)
		tProtocolCD		= self:CDTimer(45, 80053)
		-- Poison Soaked Shell
		wShell			= self:CustomWarning(L["Stop hitting %s!"], L["Warn when to stop dps on target"], 79835)
		tShellDur		= self:DurTimer(10, 79835)
		-- Bomb
		wBomb			= self:CastWarning(80157)
		tBombCD			= self:CDTimer(30, 80157)
	
	-- Nef
		-- Shadow Infusion
		self:RegisterAlert("wInfusion", self:RunawayWarning(92048))
		self:RegisterAlert("wInfusionOnYou", self:OnYouWarning(92048))
		self:RegisterAlert("iInfusion", self:Icon("SKULL", 92048, false, 15))
		self:RegisterAlert("dInfusion", self:DirectionTo(92048))
		
		self:RegisterAlert("tShaodwConductorDur", self:DurTargetTimer(10, 92053))
		
		-- Encasing Shadows
		self:RegisterAlert("wEncasing", self:RunawayWarning(92023))
		
		-- Grip
		self:RegisterAlert("wGrip", self:CastWarning(91849))
		self:RegisterAlert("tGripCast", self:CastTimer(2, 91849))
		
		-- Overcharged Generator
		self:RegisterAlert("wOvercharged", self:CastWarning(91857))
		self:RegisterAlert("tBlowBack", self:CastTimer(8, 91879))
		
		self:RegisterAlert("tNextNef", self:CustomTimer(35, L["Next Nefairan ability"], L["Timer for next Nefarian ability"], 92023, L["Nefarian Abilities"]))
	
	-- Active
	tActiveDur		= self:DurTargetTimer(120, 78740)
	wActiveChanged	= self:CustomWarning(L["Change Target!"], L["Warn when to switch target"], 78740)
end

---------------------------
-- Locals
local fixateSpam = {}

---------------------------
-- Start

function mod:OnEngage(trigger)
	-- Electron
	self:Track("Conductor", 79888)
	self:Track("UnstableShield", 79900)
	
	-- Arcanotron
	self:Track("PowerConversion", 79729)
	self:TrackCastComplete("PowerGenerator", 79624)
	
	-- Magmatron
	self:TrackAuraApplied("Acquire", 79501)
	self:Track("Barrier", 79582)
	
	-- Toxitron	
	self:Track("Bomb", 80094)
	self:TrackCastComplete("Protocol", 80053, 91514, 91515, 91513)
	self:Track("Shell", 79835, 91502, 91503)
	self:TrackCastComplete("ChemicalBomb", 80157)
	
	-- Nef
	self:Track("Infusion", 92048)
	self:Track("ShadowConductor", 92053)
	self:Track("Encasing", 92023)
	self:Track("Grip", 91849)
	self:Track("Overcharge", 91857)
	
	self:Track("GolemDeactivated", 78726)
	self:TrackCastComplete("GolemActivated",78740)
	
	wipe(fixateSpam)
	
	if self:IsHeroic() then
		self:Call("tNextNef", "Start", 20)
	end
end


-- Electron
do
	function mod:Conductor(event, args) 
		if event == "SPELL_CAST_SUCCESS" then
			if args:IsPlayer() then
				wConductor:Show()
			end
			iConductor:Show(args.dName)		
			tConductorCD:Start()
		end
	end
	
	function mod:UnstableShield(event, args)
		if event == "SPELL_AURA_APPLIED" then
			tUnstableShieldDur:Start()
		
		elseif event == "SPELL_CAST_START" then
			if UnitGUID("target") == args.sGUID and RW.Utils:GetRole() ~= "healer" then
				wUnstableShield:Show(args.sName)
			end
		end
	end
end

-- Arcanotron
do
	function mod:PowerConversion(event, args)
		if event == "SPELL_AURA_APPLIED" then
			tPowerConversionDur:Start()
		
		elseif event == "SPELL_CAST_START" then
			if UnitGUID("target") == args.sGUID and RW.Utils:GetRole() ~= "healer" then
				wPowerConversion:Show(args.sName)
			end
		end
	end
	
	function mod:PowerGenerator(args)
		wPowerGenerator:Show()
		tPowerGeneratorCD:Start()
	end
end

-- Magmatron
do
	function mod:Acquire(event, args)
		if args:IsPlayer() then
			wAcquire:Show()
		end
		iAquire:Show(args.dName)
		tAcquireCD:Start()
	end
	
	function mod:Barrier(event, args)
		if event == "SPELL_AURA_APPLIED" then
			tBarrierDur:Start()
		
		elseif event == "SPELL_CAST_START" then
			if UnitGUID("target") == args.sGUID and RW.Utils:GetRole() ~= "healer" then
				wBarrier:Show(args.sName)
			end
		end
	end
end

-- Toxitron
do
	local function ClearFixate(name)
		fixateSpam[name] = nil
	end
	function mod:Bomb(event, args)
		if event == "SPELL_CAST_SUCCESS" then
			if not fixateSpam[args.dName] then
				self:Schedule("FixateSpam"..args.dName, 5, ClearFixate, args.dName)
				
				if args:IsPlayer() then
					wFixate:Show()
				end
				
				iFixate:Show(args.dName)
				mFixate:AddDelayTarget(args.dName)
				mFixate:DelayedShow(1)
			end
		end
	end
	
	function mod:Protocol(args)
		tProtocolCD:Start()
		wProtocol:Show()
	end
	
	function mod:Shell(event, args)	
		if event == "SPELL_AURA_APPLIED" then
			tShellDur:Start()
		
		elseif event == "SPELL_CAST_START" then
			if UnitGUID("target") == args.sGUID and RW.Utils:GetRole() ~= "healer" then
				wShell:Show(args.sName)
			end
		end
	end
	
	function mod:ChemicalBomb(args)
		wBomb:Show()
		tBombCD:Start()
	end
end

-- Nef
do
	function mod:Infusion(event, args)
		if event == "SPELL_AURA_APPLIED" then
			if args:IsPlayer() then
				self:Call("wInfusionOnYou", "Show")
			else
				self:Call("wInfusion", "Show")
				self:Call("dInfusion", "Show", args.dName, 5)
			end
			
			self:Call("iInfusion", "Show", args.dName)
			self:Call("tNextNef", "Start")
		end
	end
	
	function mod:ShadowConductor(event, args)
		if event == "SPELL_AURA_APPLIED" then
			self:Call("tShaodwConductorDur", "Start", args.dName)
			
		elseif event == "SPELL_AURA_REMOVED" then
			self:Call("iInfusion", "Hide", args.dName)
			self:Call("dInfusion", "Hide", args.dName)
		end
	end
	
	function mod:Encasing(event, args)
		if event == "SPELL_AURA_APPLIED" then
			if args:IsPlayer() then
			else
				self:Call("wEncasing", "Show")
			end
			
			self:Call("tNextNef", "Start")
		end
	end
	
	function mod:Grip(event, args)
		if event == "SPELL_CAST_START" then
			self:Call("wGrip", "Show")
			self:Call("tGripCast", "Start")
			self:Call("tNextNef", "Start")
		end
	end
	
	function mod:Overcharge(event, args)
		if event == "SPELL_CAST_SUCCESS" then
			self:Call("wOvercharged", "Show")
			self:Call("tBlowBack", "Start")
			self:Call("tNextNef", "Start")
		end
	end
end

function mod:GolemActivated(args)
	if self:IsHeroic() then
		tActiveDur:Start(args.dName)
	else
		tActiveDur:Start(args.dName)
	end
	wActiveChanged:Show()
end

function mod:GolemDeactivated(event, args)
	if event == "SPELL_AURA_REMOVED" then
		tActiveDur:Stop(args.dName)
		
		if self:CUIDFromGUID(args.dGUID) == 42179 then		-- Electron
			tConductorCD:Stop()
			
		elseif self:CUIDFromGUID(args.dGUID) == 42166 then	-- Arcanotron
			tPowerGeneratorCD:Stop()
			
		elseif self:CUIDFromGUID(args.dGUID) == 42178 then	-- Magmatron
			tAcquireCD:Stop()
		
		elseif self:CUIDFromGUID(args.dGUID) == 42180 then	-- Toxitron
			tProtocolCD:Stop()
		end
	end
end
