local mod = RW:Boss("Cho'gall", "The Bastion of Twilight")
local L
local LBB

mod.BossIcon = [[Interface\ICONS\Achievement_Dungeon_Bastion of Twilight_Chogall Boss]]

---------------------------
-- Variables

---------------------------
-- Init

function mod:OnRegister()
	L = mod:RegisterLocale(self:GetName(), "enUS", true)
	if L then
	end
	L = mod:GetLocale()
	self.L = L
	
	LBB = RW.LBB
	
	self:SetTrigger(43324)
	self:SetWin()
	
	-- Summon Corrupting Adherent
	self:RegisterAlert("wAdherent", self:CastWarning(81628))
	self:RegisterAlert("tAdherentCD", self:CDTimer(92, 81628))
	
	-- Fester Blood
	self:RegisterAlert("tFesterCD", self:CDTimer(40, 82299))
	self:RegisterAlert("wFesterCast", self:CastWarning(82299))
	
	-- Warship
	self:RegisterAlert("mWarship", self:TargetMessage(91317))
	
	-- Twisted Devotion
	self:RegisterAlert("tTwistedDur", self:DurTargetStackTimer(20, 91331))
	
	-- Shadow Orders
	self:RegisterAlert("wShadowOrders", self:CastWarning(81556))
	
	-- Flame Orders
	self:RegisterAlert("wFlameOrders", self:CastWarning(81171))
	
	-- Darkened Creations
	self:RegisterAlert("wCreations", self:CastWarning(82414))
	self:RegisterAlert("tCreationCD", self:CDTimer(40, 82414))
	
	-- Fury of Cho'gall
	self:RegisterAlert("tFuryCD", self:CDTimer(48, 82524))
	self:RegisterAlert("wFury", self:CastWarning(82524))
	
	-- Chaos Blast
	self:RegisterAlert("mBlast", self:TargetStackMessage(82518))
	
	self:RegisterAlert("mPhase", self:PhaseMessage())
	
	-- Corrupting Crash
	self:RegisterAlert("iCrash", self:Icon("SKULL", 93178, false, 5))
	self:RegisterAlert("wCrash", self:RunawayWarning(93178))
	self:RegisterAlert("mCrash", self:TargetMessage(93178))
end

---------------------------
-- Locals
local CrashSpell = GetSpellInfo(93178)

---------------------------
-- Start

function mod:OnEngage(trigger)
	self:Track("Adherent", 81628)
	self:Track("Fester", 82299)
	self:Track("Warship", 91317, 93365, 93366, 93367)
	self:Track("Twisted", 91331, 93206, 93207, 93208)
	self:Track("ShadowOrders", 81556, 87575)
	self:Track("FlameOrders", 81171, 87579)
	self:Track("Creations", 82414, 93160, 93161, 93162)
	self:Track("Fury", 82524)
	self:Track("Blast", 82518, 86028, 93154, 93155, 93156)
	self:Track("Crash", 93178, 93179, 93180)
	
	self:Track("Phase2", 82630)
	
	self:Call("tAdherentCD", "Start", 52)
	self:Call("tFuryCD", "Start", 50)
end

function mod:Adherent(event, args)
	if event == "SPELL_CAST_START" then
		self:Call("wAdherent", "Show")
		self:Call("tAdherentCD", "Start")
		self:Call("tFesterCD", "Start")
	end
end

function mod:Fester(event, args)
	if event == "SPELL_CAST_START" then	
		self:Call("tFesterCD", "Stop")
		self:Call("wFesterCast", "Show")
	end
end

function mod:Warship(event, args)
	if event == "SPELL_AURA_APPLIED" then
		self:Call("mWarship", "AddDelayTarget", args.dName)
		self:Call("mWarship", "DelayedShow", 0.5)
	end
end

function mod:Twisted(event, args)
	if event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_APPLIED_DOSE" then
		if self:IsHeroic() then
			sekf:Call("tTwistedDur", "Start", 60, LBB["Cho'gall"], args.amount or 1)
		else
			self:Call("tTwistedDur", "Start", nil, LBB["Cho'gall"], args.amount or 1)
		end
	end
end

function mod:ShadowOrders(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		self:Call("wShadowOrders", "Show")
	end
end

function mod:FlameOrders(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		self:Call("wFlameOrders", "Show")
	end
end

function mod:Creations(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		self:Call("wCreations", "Show")
		self:Call("tCreationCD", "Start")
	end
end

function mod:Phase2(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		self:Call("mPhase", "Show", 2)
		
		self:Call("tFesterCD", "Stop")
		self:Call("tAdherentCD", "Stop")
	end
end

function mod:Fury(event, args)
	if event == "SPELL_CAST_START" then
		self:Call("tFuryCD", "Start")
		self:Call("wFury", "Show")
	end
end

function mod:Blast(event, args)
	if event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_APPLIED_DOSE" then
		self:Call("mBlast", "Show", args.dName, args.amount or 1)
	end
end

function mod:CrashTarget()
	local target = self:GetMobTarget(43622)
	if target then
		if target == self.pName then
			self:Call("wCrash", "Show")
			SendChatMessage(CrashSpell.." on "..self.pName.."!", "SAY")
		elseif self.Party.group[target] then
			local uID = self.Party.group[target].id
			if uID and RW.Utils:IsUnitInRange(uID, 10) then
				self:Call("wCrash", "Show")
			end
		end
		self:Call("mCrash", "Show", target)
		self:Call("iCrash", "Show", target)
	end
end

function mod:Crash(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		self:Schedule("CrashSchedule", 0.2, "CrashTarget")
	end
end
